// Arduino UNO用Scratch対応ファームウエア
// 注意：専用のヘルパーアプリが必要です
// by Takao Takahashi
// https://tsq.gapj.net

//Arduino UNOのピン定義
static uint8_t digitalPins[] = {2,3,4,5,6,7,8,9,10,11,12,13}; //デジタルピン列挙
static uint16_t analogPins[] ={0,1,2,3,4,5}; //アナログピン列挙
//ピンの動作中モードを覚えておく
uint8_t skipmePins[16]; //analogWriteしたピンは読み出さないためのモードを覚えておく
                        //(PWM出力したピンをdigitalRead()すると出力が停止するため)

//コマンド文字列とその値用
//ヘルパーからの電文はA1 255のような形なので32バイトもあれば十分
char sbuffer[32];
//コマンドから指定されるピンとその値
uint8_t cmd_pin, cmd_value;
//ピン列挙の配列のサイズ格納用
uint8_t a_maxDigital;
uint8_t a_maxAnalog;

//初期化処理
void setup() {
  //シリアルポート初期化
  Serial.begin(115200);
  Serial.flush();
  //ピン配列のサイズをセット
  a_maxDigital = sizeof(digitalPins) / sizeof(uint8_t);
  a_maxAnalog = sizeof(analogPins) / sizeof(uint16_t);
}


//メインループ
void loop() {
  reportAllPins(); //全ピン読み出してシリアルへ送出
  sreadCommand();  //ヘルパーからのコマンド処理
}
//メインループはここまで

//ピンの値の読み込みとヘルパーアプリへの送信
//Scratchのリポーターブロックが使う値を送信する
void reportAllPins(){
  uint8_t i;
  uint16_t value;
  uint8_t a_max;

  //デジタルピンの読み込み
  for(i=0;i<a_maxDigital;i++){
    if(skipmePins[i] != 'A'){ //そのピンがアナログ(PWM)モードなら読み出さない
      value = digitalRead(digitalPins[i]);
    }
    Serial.print("D");
    Serial.print(digitalPins[i]);
    Serial.print(" ");
    Serial.print(value);
    Serial.print(":");
  }
  //アナログピンの読み込み
  //A/D変換の取り込み
  //Arduinoでは処理せずヘルパー(リポーターブロック)へ送る
  for(i=0;i<a_maxAnalog;i++){
    value = analogRead(analogPins[i]);
    Serial.print("A");
    Serial.print(analogPins[i]);
    Serial.print(" ");
    Serial.print(value);
    Serial.print(":");
  }
  Serial.print("\n");
}

//シリアルからのコマンド受信
//ヘルパーからの動作指示を処理する
void sreadCommand(){
  uint8_t i;
  char tmp_c;

  if(Serial.available()){ //受信した文字があれば処理する
     i = 0;
     while(1){ //文字列受信完了までのループ
       if(Serial.available()){ //受信した文字がある
         tmp_c = Serial.read(); //1文字読み出し
         if(tmp_c == 0x0a) break; //行末はCRかLF
         if(tmp_c == 0x0d) break; //CRかLFを受信したら1行完了
         sbuffer[i] = tmp_c;
         sbuffer[i+1] = 0x00; //ひとつ先を0x00にしておいてバッファ初期化をさぼる
         i++;
         if(i>30) break; //終端(CRorLF)されず30文字越えたらやめる
       }
     }
     //受信したコマンドの処理
     //コマンドの文字(頭の1バイト)によって処理を切り替える
     switch(sbuffer[0]){
       case 'D': //デジタル出力
         digitalOut();
         break;
       case 'A': //アナログ出力
         analogOut();
         break;
       case 'M': //ピンモード設定
         pinmSet();
         break;
       default : //該当するコマンドがなければ何もしない
         break;
     }
   } //何も受信していなければそのまま戻る
}


//デジタルポートへの出力処理
//High/Lowのデジタル出力
void digitalOut()
{
  uint8_t i;
  
  cmd2Value();
  for(i=0;i<a_maxDigital;i++){ //デジタルモードで動作中をセットする
    if(digitalPins[i] == cmd_pin) skipmePins[i] = 'D';
  }
  //各値(0,1)に応じて出力する
  if(cmd_value == 0){
    digitalWrite(cmd_pin, LOW); 
  }
  if(cmd_value == 1){
    digitalWrite(cmd_pin, HIGH);
  }
}

//アナログ出力(PWM)
//analogWriteによるデジタルピンのPWM出力
void analogOut()
{
  uint8_t i;
  
  cmd2Value();
  for(i=0;i<a_maxDigital;i++){ //アナログモードで動作中をセットする
    if(digitalPins[i] == cmd_pin) skipmePins[i] = 'A';
  }
  //指定された値を出力する
  analogWrite(cmd_pin, cmd_value);
}

//ピンモード設定(INPUT or OUTPUT)
void pinmSet()
{
  cmd2Value();
  if(cmd_value == 0){
    pinMode(cmd_pin, INPUT);
  }
  if(cmd_value == 1){
    pinMode(cmd_pin, OUTPUT);
  }
}

//コマンド文字列から値へ変換する関数
//コマンドの文字を整数型に変換する
void cmd2Value(){
  cmd_pin = atoi(strtok(sbuffer+1," "));
  cmd_value = atol(strtok(NULL," "));
}
